{-# LANGUAGE Arrows #-}

module Main where

import FRP.YFrob.RobotSim

------------------------------------------------------------------------------
-- Controllers for team 1 here
------------------------------------------------------------------------------

player1_1 :: SimbotController
player1_1 = rcStop


player1_2 :: SimbotController
player1_2 = rcGoToBall2 1.5


player1_3 :: SimbotController
player1_3 = rcStop


------------------------------------------------------------------------------
-- Controllers for team 2 here
------------------------------------------------------------------------------

player2_1 :: SimbotController
player2_1 = rcStop


player2_2 :: SimbotController
player2_2 = rcGoToBall2 0.5


player2_3 :: SimbotController
player2_3 = rcStop


------------------------------------------------------------------------------
-- Some potentially useful controllers
------------------------------------------------------------------------------

-- Goes towards the ball.
rcGoToBall :: Velocity -> SimbotController
rcGoToBall vd rps = proc sbi -> do
    let (phi, d) = head (aotBalls sbi ++ [(0.0,0.0)])
        h        = odometryHeading sbi
    rcHeadingAux rps -< (sbi, vd, h + phi)


rcGoToBall2 :: Velocity -> SimbotController
rcGoToBall2 vd rps =
    let loop = switch (rcGoToBall vd rps &&& rsStuck) $ \_ ->
               switch (constant (mrFinalize (ddVelTR (-vd) 0.3))
                       &&& after 2.5 ()) $ \_ ->
               loop
    in
        loop


-- Goes in the desired heading at the desired speed.
rcHeadingAux :: SimbotProperties -> 
                SF (SimbotInput,Velocity,Heading) SimbotOutput
rcHeadingAux rps = 
    let vMax = rpWSMax rps
        k    = 2
    in proc (sbi,vel,hd) -> do
         let vel' = lim vMax vel
         let phi = normalizeAngle (hd - odometryHeading sbi)
         let vel'' = (1 - abs phi / pi) * vel'
         returnA -< mrFinalize (ddVelTR vel'' (k*phi))

lim m y = max (-m) (min m y)


-- Stops the robot.
rcStop :: SimbotController
rcStop _ = constant (mrFinalize ddBrake)


------------------------------------------------------------------------------
-- Main
------------------------------------------------------------------------------

main = playSoccer [player1_1, player1_2, player1_3]
                  [player2_1, player2_2, player2_3]
